package com.xueyi.system.digitalmans.manager;

import com.xueyi.system.api.digitalmans.domain.dto.DmManDeviceDto;
import com.xueyi.system.digitalmans.domain.query.DmManDeviceQuery;
import com.xueyi.common.web.entity.manager.IBaseManager;

import java.util.List;

/**
 * 数字人设备管理管理 数据封装层
 *
 * @author xueyi
 */
public interface IDmManDeviceManager extends IBaseManager<DmManDeviceQuery, DmManDeviceDto> {
    DmManDeviceDto selectInfoByDevId(String devId);
    DmManDeviceDto selectInfoByManCode(String devId);

    Integer updateInfoByManual(DmManDeviceDto dto);

    List<DmManDeviceDto> selectListByManual(DmManDeviceQuery query);
    List<DmManDeviceDto> selectManDeviceListNotNullByTenantId(Long tenantId);
}